import { __spreadArrays } from "tslib";
import { fs } from 'memfs';
import path from 'path';
import asc from 'assemblyscript/cli/asc';
import loaderUtils from 'loader-utils';
import validate from 'schema-utils';
import { schemaOptions, transOptionToAscOption } from './option';
var loaderpkg = '@assemblyscript/loader';
var configuration = {
    name: 'assemblyscript-loader-loader',
};
var getInBundleWasmModule = function (buf) { return "\nimport loader from '" + loaderpkg + "';\nexport default async (imports) => {\n  return await loader.instantiate(" + buf.buffer + ", imports || {});\n};"; };
var getFileWasmModule = function (fetchUrl) { return "\nimport loader from '" + loaderpkg + "';\nexport default async (imports) => {\n  return await loader.instantiateStreaming(fetch(" + fetchUrl + "), imports || {});\n}"; };
export default (function (content) {
    var options = loaderUtils.getOptions(content) || {};
    var callback = content.async();
    // eslint-disable-next-line
    var buildPath = content._compiler.outputPath;
    var output = path.join(buildPath, path.parse(content.resourcePath).name + ".wasm");
    var ascArgv = __spreadArrays([
        path.relative(process.cwd(), content.resourcePath),
        '-o', path.relative(process.cwd(), output)
    ], transOptionToAscOption(options));
    validate(schemaOptions, options, configuration);
    content.addDependency(loaderpkg);
    asc.ready.then(function () {
        asc.main(ascArgv, {
            readFile: function (filename, baseDir) {
                var filePath = path.join(baseDir, filename);
                console.log('filePath-read', filePath);
                return fs.readFileSync(filePath).toString();
            },
            writeFile: function (filename, contents, baseDir) {
                var filePath = path.join(baseDir, filename);
                console.log('filePath-write', filePath);
                fs.writeFileSync(filePath, contents);
            },
        }, function (err) {
            if (err) {
                callback(err);
                return 0;
            }
            var size = fs.statSync(output).size;
            var wasmFile = fs.readFileSync(output);
            // 默认 1000
            var limit = Number(options.limit) || 1000;
            if (size <= limit) {
                callback(null, getInBundleWasmModule(Buffer.from(wasmFile)));
            }
            else {
                var fileName = loaderUtils.interpolateName(content, typeof options.name === 'string' && options.name
                    ? options.name
                    : '[name].[hash:7].wasm', {
                    context: content.rootContext,
                    content: wasmFile,
                });
                content.emitFile(fileName, wasmFile, undefined);
                var fetchUrl = "__webpack_public_path__ + " + JSON.stringify(fileName);
                if (options.publicPath) {
                    fetchUrl = JSON.stringify("" + options.publicPath + fileName);
                }
                else if (options.useInWorker) {
                    fetchUrl = "self.location.origin + '/" + fileName + "'";
                }
                callback(null, getFileWasmModule(fetchUrl));
            }
            return 0;
        });
    });
});
//# sourceMappingURL=index.js.map